﻿-- Arena Style for compact raid frame more buffs and debuffs, and show debuffs outside of the unit frame


-- Initialized database defaults
local defaults = {enable = true,buffnum = 6,debuffnum = 10,min = 5}

ArenaStyleDB = ArenaStyleDB or defaults
ArenaStyle = CreateFrame("Frame")
ArenaStyle.cache = {}

-- default buff number
local DEFAULT_BUFF = 3

-- default debuff number
local DEFAULT_DEBUFF = 3

local function log(msg) DEFAULT_CHAT_FRAME:AddMessage("|cffCD32CDArenaStyle:|r " .. msg) end

function ArenaStyle:OnLoad()
	-- Initialize the db
	self.db = ArenaStyleDB
	for k,v in pairs(defaults) do 
		if self.db[k] == nil then self.db[k] = defaults[k] end
	end
	-- Initialize command table
	SlashCmdList["ArenaStyle"] = function(msg) self:Command(msg, tbl) end
	SLASH_ArenaStyle1 = "/as"
	SLASH_ArenaStyle2 = "/ArenaStyle"
	log("type /as to config ArenaStyle.")
	-- Register event
	--self:RegisterEvent("RAID_ROSTER_UPDATE")
	--self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	--self:RegisterEvent("UNIT_PET")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("UNIT_AURA")
	CompactRaidFrameContainer:HookScript("OnEvent",ArenaStyle.OnRosterUpdate)
	CompactRaidFrameContainer:HookScript("OnHide",ArenaStyle.ResetStyle)
	CompactRaidFrameContainer:HookScript("OnShow",ArenaStyle.OnRosterUpdate)
end

-- Command table
function ArenaStyle:Command(msg , tbl)
	local cmdtbl = tbl or 
	{
		["buff"] = function(arg)
			if tonumber(arg) and tonumber(arg) >= DEFAULT_BUFF then 
				self.db.buffnum = tonumber(arg) 
				log("buff number set to be "..self.db.buffnum) 
				self:ResetStyle() 
				self:OnRosterUpdate()
			else 
				log("invalid input(must be numbers and over 3)") 
			end
		end,
		["debuff"] = function(arg) 
			if tonumber(arg) and tonumber(arg) >= DEFAULT_DEBUFF then 
				self.db.debuffnum = tonumber(arg) 
				log("debuff number set to be "..self.db.debuffnum)
				self:ResetStyle() 
				self:OnRosterUpdate()
			else
				log("invalid input(must be numbers and over 3)") 
			end
		end,
		["min"] = function(arg) 
			if tonumber(arg) and tonumber(arg) >= 0 then 
				self.db.min = tonumber(arg) 
				log("the minimum number to apply arena style set to be "..self.db.min) 
				self:OnRosterUpdate() 
			else 
				log("invalid input(must be numbers and non-negative)") 
			end 
		end,
		["off"] = function() 
			self.db.enable = false
			log("set to be disabled") 
			self:ResetStyle()
		end,
		["on"] = function() 
			self.db.enable = true
			log("set to be enabled") 
			self:OnRosterUpdate()
		end,
		--["apply"] = function() self:ApplyStyle() log("arena style applied") end,
		["status"] = function() log ("addon[".. (self.db.enable and "on]" or "off]").."  buff["..self.db.buffnum.."]".."  debuff["..self.db.debuffnum.."]".."  min["..self.db.min.."]") end,
		--["reset"] = function() self:ResetStyle() log("reset to default style") end,
		--["reset"] = function() ArenaStyleDB = {}; log("reset ArenaStyle's database") end,
 		["help"] = function() log("\n'/as debuff [number]' to set debuff number\n'/as buff [number]' to set buff number\n'/as min [number]' to set the minimum number of party or raid members to auto apply arena style\n'/as [on/off]' to enable or disable the addon") end,
		-- \n'/as status' to see the status\n'/as reset' to reset database(works after reloading)
	}
	local cmd, arg = string.split(" ", msg, 2)
	local entry = cmdtbl[cmd:lower()]
	local which = type(entry)
	if which == "function" then
		entry(arg)
	elseif which == "table" then
		self:Command(arg or "" , entry)
	else
		self:Command("help")
	end
end
--[[function CompactRaidFrameContainer_OnEvent(self, event, ...)
    if ( event == "RAID_ROSTER_UPDATE" or event == "PARTY_MEMBERS_CHANGED" ) then
        CompactRaidFrameContainer_UpdateDisplayedUnits(self);
        CompactRaidFrameContainer_TryUpdate(self);
    elseif ( event == "UNIT_PET" ) then
        if ( self.displayPets ) then
            local unit = ...;
            if ( unit == "player" or strsub(unit, 1, 4) == "raid" or strsub(unit, 1, 5) == "party" ) then
                CompactRaidFrameContainer_TryUpdate(self);
            end
        end
    end
end]]
-- When roster updated, auto apply arena style or reset style
function ArenaStyle:OnRosterUpdate()
	local _,areaType = IsInInstance()
	if areaType == "raid" then ArenaStyle:ResetStyle() return end
	if not ArenaStyle.db.enable then return end
	if not CompactRaidFrameContainer:IsVisible() then return end
	local n = max (GetNumRaidMembers(), GetNumPartyMembers())
	if n <= ArenaStyle.db.min and n > 0 then ArenaStyle:ApplyStyle() else ArenaStyle:ResetStyle() end
end

-- If in raid reset style
function ArenaStyle:OnZoneChanged()
	local _,areaType = IsInInstance()
	self:ResetStyle()
	if areaType ~= "raid" then self:ApplyStyle() end
end

-- Apply arena style to the compact raid frame
function ArenaStyle:ApplyStyle()
	for i = 1,80 do 
		local f = _G["CompactRaidFrame"..i]
		if f and not self.cache[f] and f.inUse and f.maxBuffs ~= 0 and #f.buffFrames == DEFAULT_BUFF and f.unit and not strfind(f.unit,"pet") and not strfind(f.unit,"target") then
			self:ApplyFrame(f)
			self:UpdateAura(f.unit)
		end
		if f and not f.inUse and self.cache[f] then
			self:ResetFrame(f)
		end
	end
end
-- Update aura for each unit
function ArenaStyle:UpdateAura(uid)
	for f,v in pairs(self.cache) do
		if f.unit == uid then
			local filter = nil
			local buffNum = 1
			local index = 1
			while buffNum <= self.db.buffnum do
				local buffName = UnitBuff(uid, index, filter);
				if ( buffName ) then
					if ( CompactUnitFrame_UtilShouldDisplayBuff(uid, index, filter) ) then
						if buffNum > DEFAULT_BUFF then
							local buffFrame = v.buffFrames[buffNum]
							CompactUnitFrame_UtilSetBuff(buffFrame, uid, index, filter)
						end
						buffNum = buffNum + 1
					end
				else
					break
				end
				index = index + 1
			end
			for i=buffNum, self.db.buffnum do
				local buffFrame = v.buffFrames[i]
				if buffFrame then
					buffFrame:Hide()
				end
			end
			local debuffNum = 1
			index = 1
			while debuffNum <= self.db.debuffnum do
				local debuffName = UnitDebuff(uid, index, filter)
				if ( debuffName ) then
					if ( CompactUnitFrame_UtilShouldDisplayDebuff(uid, index, filter) ) then
						local debuffFrame = v.debuffFrames[debuffNum]
						CompactUnitFrame_UtilSetDebuff(debuffFrame, uid, index, filter)
						debuffNum = debuffNum + 1
					end
				else
					break
				end
				index = index + 1
			end
			for i=debuffNum, self.db.debuffnum do
				local debuffFrame = v.debuffFrames[i];
				if debuffFrame then
					debuffFrame:Hide()
				end
			end
			break
		end
	end
end
-- Apply style for each frame
function ArenaStyle:ApplyFrame(f)
	self.cache[f] = {}
	f:SetScript("OnSizeChanged",function() ArenaStyle:ResetFrame(f) ArenaStyle:ApplyFrame(f) end) 
	if not self.cache[f].buffFrames then self.cache[f].buffFrames = {} end
	if not self.cache[f].debuffFrames then self.cache[f].debuffFrames = {} end
	for j = DEFAULT_BUFF + 1, self.db.buffnum do
		if not self.cache[f].buffFrames[j] then
			self.cache[f].buffFrames[j] = CreateFrame("Button",nil,UIParent,"ArenaStyleBuffTemplate")
			self.cache[f].buffFrames[j].unit = f.unit
			--self.cache[f].buffFrames[j]:EnableMouse(false)
			if j == DEFAULT_BUFF + 1 then
				self.cache[f].buffFrames[j]:SetPoint("RIGHT",f.buffFrames[3],"LEFT")
			else
				self.cache[f].buffFrames[j]:SetPoint("RIGHT",self.cache[f].buffFrames[j-1],"LEFT")
			end
			self.cache[f].buffFrames[j]:SetSize(f.buffFrames[3]:GetSize())
		end
	end
	for j = 1,self.db.debuffnum do
		if not self.cache[f].debuffFrames[j] then
			self.cache[f].debuffFrames[j] = CreateFrame("Button",nil,UIParent,"ArenaStyleDebuffTemplate")
			self.cache[f].debuffFrames[j].unit = f.unit
			--self.cache[f].debuffFrames[j]:EnableMouse(false)
			if j == 1 then
				self.cache[f].debuffFrames[j]:ClearAllPoints()
				self.cache[f].debuffFrames[j]:SetPoint("BOTTOMLEFT", f, "BOTTOMRIGHT",1,10) 
			else
				self.cache[f].debuffFrames[j]:SetPoint("LEFT",self.cache[f].debuffFrames[j-1],"RIGHT")
			end
			--f.debuffFrames[j]:SetSize(f.buffFrames[3]:GetSize())
			self.cache[f].debuffFrames[j]:SetSize(f.buffFrames[3]:GetSize())
			self.cache[f].debuffFrames[j]:Hide()
		end
	end
	for j = 1,#f.debuffFrames do
		f.debuffFrames[j]:Hide()
		f.debuffFrames[j]:SetScript("OnShow",f.debuffFrames[j].Hide) 
	end
end
-- Reset to the original style
function ArenaStyle:ResetStyle()
	for f,_ in pairs(ArenaStyle.cache) do
		ArenaStyle:ResetFrame(f)
	end
end
-- Reset style to each cached frame
function ArenaStyle:ResetFrame(f)
	for k,v in pairs(self.cache[f].buffFrames) do
		if v then
			v:Hide()
		end
	end
	for k,v in pairs(self.cache[f].debuffFrames) do
		if v then
			v:Hide()
		end
	end
	f:SetScript("OnSizeChanged",nil)
	for j = 1,#f.debuffFrames do
		f.debuffFrames[j]:SetScript("OnShow",nil) 
	end
	self.cache[f] = nil
end
function ArenaStyle:OnCombat()
	-- todo
end
function ArenaStyle:LeaveCombat()
	-- todo
end

-- Event handling
local function OnEvent(self,event,...)
	if event == "VARIABLES_LOADED" then self:OnLoad() end
	if event == "RAID_ROSTER_UPDATE" or event == "PARTY_MEMBERS_CHANGED" or event == "UNIT_PET" then self:OnRosterUpdate() end
	if event == "PLAYER_ENTERING_WORLD" then self:OnRosterUpdate() end
	if event == "PLAYER_REGEN_DISABLED" then self:OnCombat() end
	if event == "PLAYER_REGEN_ENABLED" then self:LeaveCombat() end
	if event == "UNIT_AURA" then self:UpdateAura(...) end
end

ArenaStyle:SetScript("OnEvent",OnEvent)
ArenaStyle:RegisterEvent("VARIABLES_LOADED")
_G.ArenaStyle = ArenaStyle